<?php
/**
* CalDAV Server - handle DELETE method
*
* @package   davical
* @subpackage   caldav
* @author    Andrew McMillan <andrew@mcmillan.net.nz>
* @copyright Catalyst .Net Ltd, Morphoss Ltd <http://www.morphoss.com/>
* @license   http://gnu.org/copyleft/gpl.html GNU GPL v2
*/
dbg_error_log("delete", "DELETE method handler");

/**
* etag_none_match, if set, is telling us only to DELETE if it fails to match.  Likewise etag_if_match
* is telling us only to DELETE if it successfully matches the ETag.  Se Evolution's Bugzilla for the
* truth about Evolution's broken handling of this: http://bugzilla.gnome.org/show_bug.cgi?id=349573
*/

if ( !isset($request->etag_if_match) && isset($request->etag_none_match) && isset($_SERVER['HTTP_USER_AGENT'])
          && preg_match('#Evolution/([0-9]+[.][0-9]+)#', $_SERVER['HTTP_USER_AGENT'], $matches ) ) {
  if ( doubleval($matches[1]) <= 1.9 ) {
    $request->etag_if_match = $request->etag_none_match;
    unset($request->etag_none_match);
  }
}

$request->NeedPrivilege('DAV::unbind');

$lock_opener = $request->FailIfLocked();

if ( $request->IsCollection() ) {
  /**
  * We read the collection first, so we can check if it matches (or does not match)
  */
  $qry = new AwlQuery( 'SELECT * FROM collection WHERE user_no = :user_no AND dav_name = :dav_name',
              array( ':user_no' => $request->user_no, ':dav_name' => $request->path ) );
  if ( $qry->Exec('DELETE',__LINE__,__FILE__) && $qry->rows() == 1 ) {
    $delete_row = $qry->Fetch();
    if ( (isset($request->etag_if_match) && $request->etag_if_match != $delete_row->dav_etag) ) {
      $request->DoResponse( 412, translate("Resource does not match 'If-Match' header - not deleted") );
    }

    $path_like = array( ':path_like' => $request->path.'%' );
    if ( $qry->Begin()
         && $qry->QDo("SELECT write_sync_change(collection_id, 404, caldav_data.dav_name) FROM caldav_data WHERE dav_name LIKE :path_like", $path_like )
         && $qry->QDo("DELETE FROM collection WHERE dav_name = :request_path", array(':request_path' => $request->path) )
         && $qry->QDo("DELETE FROM caldav_data WHERE dav_name LIKE :path_like", $path_like )
         && $qry->QDo("DELETE FROM property WHERE dav_name LIKE :path_like", $path_like )
         && $qry->QDo("DELETE FROM locks WHERE dav_name LIKE :path_like", $path_like )
         && $qry->Commit() ) {
      @dbg_error_log( "DELETE", "DELETE (collection): User: %d, ETag: %s, Path: %s", $session->user_no, $request->etag_if_match, $request->path);
      $request->DoResponse( 204 );
    }
    else {
      $request->DoResponse( 500, translate("Error querying database.") );
    }

  }
  else {
    $request->DoResponse( 404 );
  }
}
else {
  /**
  * We read the resource first, so we can check if it matches (or does not match)
  */
  $escaped_path = qpg($request->path);
  $qry = new AwlQuery( "SELECT cd.dav_etag, ci.uid, cd.collection_id FROM caldav_data cd JOIN calendar_item ci USING (dav_id) WHERE cd.dav_name = $escaped_path" );
  if ( $qry->Exec("DELETE") && $qry->rows() == 1 ) {
    $delete_row = $qry->Fetch();
    if ( (isset($request->etag_if_match) && $request->etag_if_match != $delete_row->dav_etag) ) {
      $request->DoResponse( 412, translate("Resource has changed on server - not deleted") );
    }

    $collection_id = $delete_row->collection_id;

    if ( $qry->Begin()
         && $qry->QDo("DELETE FROM caldav_data WHERE collection_id = $collection_id AND dav_name = $escaped_path" )
         && $qry->QDo("SELECT write_sync_change( $collection_id, 404, $escaped_path)" )
         && $qry->QDo("DELETE FROM property WHERE dav_name = $escaped_path" )
         && $qry->Commit() ) {
      @dbg_error_log( "DELETE", "DELETE: User: %d, ETag: %s, Path: %s", $session->user_no, $request->etag_if_match, $request->path);
      if ( function_exists('log_caldav_action') ) {
        log_caldav_action( 'DELETE', $delete_row->uid, $request->user_no, $request->collection_id, $request->path );
      }
      $request->DoResponse( 204 );
    }
    else {
      $request->DoResponse( 500, translate("Error querying database.") );
    }
  }
  else {
    $request->DoResponse( 404 );
  }
}

